/*
 *
 * OH2NLT Iambic / Golden Key keyer
 *
 * OH2NLT, Juha Niinikoski 29.12.2005
 *
 * PIC10F206 6-pin microcontroller
 * Hi-Tech PICC compiler v8.05PL2 & MPLAB7.00
 *
 *
 * First release version 1.00 30.12.2005
 * Straight action from DA switch 1.01 01.01.2006
 *
*/

#include <pic.h>
//#include <pic10f20x.h>

	__CONFIG(WDTDIS & MCLRDIS);						// WDT & MCLR disable

const char id[] = {"Juha Niinikoski, OH2NLT  01.01.2006"};	// Visible id string

unsigned char gp_copy;		// GP port copy
bit sound;					// sound on flag for unit delay routine
bit gk;						// period precedence, Golden Key type action
bit straight;				// straight action from DA switch

// wait routine variables
unsigned char wctr;			// wait counter, decrement from set value to 0
bit wphase;					// wait routine phase

#define DI	GP2				// key inputs
#define DA	GP3
#define KEY GP0				// key output to radio
#define SOUND_OUT GP2		// BUZZER sound output

#define READ_SW		0x0C	// GP2 = input
#define BUZZ_OUT	0x08	// GP2 = output

#define OPT_INIT	0xD2	// prescaler = 8, pins for GPIO, 125kHz for TMR0

// GP port bit masks
#define LOAD_CAP	0x02	// load cap pin mask
#define TX_ON		0x01	// TX (key) signal to radio
#define SOUND_ON	0x04	// buzzer out

// sound
#define TONE_MASK	0x40	// TMR0 bit mask = audio frequency pitcj
#define DEBOUNCE	20		// straight mode key switch debounce (ms)


// Remember, we have only one free running timer
// without any flags. No interrupts.

// 1,016mS delay
// using free running TMR0, 125kHz
void ms_delay(unsigned char ms)
	{
	unsigned char x;
	for(x=0; x<ms; x++)
		{
		while((TMR0 & 0x40) == 0);	// wait low phase
		while((TMR0 & 0x40) != 0);	// wait high phase
		}
	}


// 1,016mS wait, decrement wctr until 0
// using free running TMR0, 125kHz
void run_wait(void)
	{
	if(wctr == 0)
		return;						// run only when wctr != 0


	if(wphase)						// wphase = 1
		{
		if((TMR0 & 0x80) == 0)		// wait TMR D7 go low
			{
			wphase = 0;
			wctr--;
			}

		}
	else							// wphase = 0
		{
		if((TMR0 & 0x80) != 0)		// wait TMR D7 go high
			{
			wphase = 1;
			wctr--;
			}
		}
	}

// Comparator RC unit delay routine
// make also sound if sound flag set

void unit_delay(void)
	{
	char x;
	CMPON = 0;						// I(O pin to GP port, start charge
	for(x=0; x<10; x++)				// charge delay xx us for 1u cap
		{
		if(sound == 1)				// generate sound
			{
			if(TMR0 & TONE_MASK)	// D6 = buzzer phase
				{
				gp_copy = gp_copy | SOUND_ON;
				GPIO = gp_copy;		// set buzzer output
				}
			else
				{
				gp_copy = gp_copy & ~SOUND_ON;
				GPIO = gp_copy;		// reset buzzer output
				}
			}
		else						// just wait for unit delay end
			{
			asm("nop");				// keep charging time equal with sound on 
			asm("nop");				// charging time
			asm("nop");
			asm("nop");
			asm("nop");
			asm("nop");
			asm("nop");
			}
		}

	CMPON = 1;						// stop cap loading & start new cycle

	asm("nop");						// comparator start up delay
	asm("nop");	


	if(sound == 1)					// generate sound
		{
		while(CMPOUT == 1)			// wait for delay end, generate sound if required
			{
			if(TMR0 & TONE_MASK)	// D6 = buzzer phase
				{
				gp_copy = gp_copy | SOUND_ON;
				GPIO = gp_copy;		// set buzzer output
				}
			else
				{
				gp_copy = gp_copy & ~SOUND_ON;
				GPIO = gp_copy;		// reset buzzer output
				}
			}
		}
	else							// just wait for unit delay end
		while(CMPOUT == 1);	
			{
			}
	}


// say hello with R .-. and mode I .. or G --. or S ...
// not optimized

void hello(void)
	{
	TRIS = BUZZ_OUT;						// switch GP2 for sound output

	gp_copy = gp_copy |  SOUND_ON;			// start sound with high state, key not pressed
	GPIO = gp_copy;
	TMR0 = 0x40;							// sync timer to high start

#if 0
// R
	sound = 1;								// make sound
	unit_delay();							// one unit for DI
	sound = 0;
	unit_delay();
	sound = 1;								// make sound
	unit_delay();							// three units for DAH
	unit_delay();
	unit_delay();
	sound = 0;
	unit_delay();
	sound = 1;								// make sound
	unit_delay();							// one unit for DI
	sound = 0;
	unit_delay();
	unit_delay();
	unit_delay();
#endif

	if((gk == 0) && (straight == 0))		// Iambic mode
		{
// I
		sound = 1;							// make sound
		unit_delay();						// one unit for DI
		sound = 0;
		unit_delay();
		sound = 1;							// make sound
		unit_delay();						// one unit for DI
		sound = 0;
		unit_delay();
		unit_delay();
		unit_delay();
		}
	else
		{
		if(gk)								// Golden Key mode
			{
// G
			sound = 1;						// make sound
			unit_delay();					// one unit for DAH
			unit_delay();
			unit_delay();
			sound = 0;
			unit_delay();
			sound = 1;						// make sound
			unit_delay();					// one unit for DAH
			unit_delay();
			unit_delay();
			sound = 0;
			unit_delay();
			sound = 1;						// make sound
			unit_delay();					// one unit for DI
			sound = 0;
			unit_delay();
			unit_delay();
			unit_delay();
			}
		else								// straight mode
			{
// S
			sound = 1;						// make sound
			unit_delay();					// one unit for DI
			sound = 0;
			unit_delay();
			sound = 1;						// make sound
			unit_delay();					// one unit for DI
			sound = 0;
			unit_delay();
			sound = 1;						// make sound
			unit_delay();					// one unit for DI
			sound = 0;
			unit_delay();
			unit_delay();
			unit_delay();
			}
		}

	gp_copy = gp_copy |  SOUND_ON;	// leave buzzer signal to high state
	GPIO = gp_copy;
	sound = 0;
	unit_delay();					// exit delay

	TRIS = READ_SW;					// back to key read mode
	unit_delay();					// I/O stable delay
	}


// main control loop

void main(void)
{
char x;

// init I/O

	OPTION = OPT_INIT;				// pins for GP I/O
	TRIS = READ_SW;					// GP2, GP3 = inputs
	CMCON = 0x79;					// comparator on, pos ref = GP1 pin, pol true
	FOSC4 = 0;						// GP2 for I/O
	KEY = 0;						// TX off
	gp_copy = LOAD_CAP;				// CAP load bit = 1;
	GPIO = gp_copy;					// set GPIO port



// set keyer mode

	if(DI == 0)						// set Golde Key mode
		gk = 1;						// if DI key pressed at startup select Golden Key mode

	if(DA == 0)						// set straight mode
		straight = 1;				// if DA key pressed at startup select straight mode

	hello();						// say hello with Morse R and I, G or S
	while(DI == 0 || DA == 0);		// wait for key release


#if 0
	for(;;)		// delay test
		{
		ms_delay(100);
		KEY=1;
		ms_delay(10);
		KEY=0;
		}

	for(;;)		// wait test
		{
		if(wctr == 0)
			{
			KEY=!KEY;
			wctr = 10;
			}
		run_wait();
		}
#endif


// Main forever loop

	for(;;)
		{
Manual:

		if(DA == 0)							// generate DAH
			{
			if(straight)					// check if straight mode
				{
				TRIS = BUZZ_OUT;			// switch GP2 for sound output
				TMR0 = 0;					// sync timer for sound
				gp_copy = gp_copy | TX_ON;	// start TX
				wctr = DEBOUNCE;			// set debounce

				while(DA == 0 || ((gp_copy & SOUND_ON) == 0) || (wctr != 0))	// stay here as long as key pressed
					{
					if(TMR0 & TONE_MASK)				// D6 = buzzer phase
						{
						gp_copy = gp_copy | SOUND_ON;
						GPIO = gp_copy;					// set buzzer output
						}
					else
						{
						gp_copy = gp_copy & ~SOUND_ON;
						GPIO = gp_copy;					// reset buzzer output
						}

						if(wctr != 0)					// run wait timer in line, better audio quality
							{
							if(wphase)					// wphase = 1
								{
								if((TMR0 & 0x80) == 0)	// wait TMR D7 go low
								{
								wphase = 0;
								wctr--;
								}
							}
						else							// wphase = 0
							{
							if((TMR0 & 0x80) != 0)		// wait TMR D7 go high
								{
								wphase = 1;
								wctr--;
								}
							}
						}								// end while
					}
				gp_copy = gp_copy & ~TX_ON;	// stop TX
				GPIO = gp_copy;
				TRIS = READ_SW;				// back to key read mode
				ms_delay(DEBOUNCE);			// debounce / C5 load time.
				}
			else					// auto DAH
				{
			gp_copy = gp_copy | TX_ON & ~SOUND_ON;	// start TX, 0 to buzzer line = less click
			GPIO = gp_copy;
			TRIS = BUZZ_OUT;		// switch GP2 for sound output
			TMR0 = 0;				// sync timer for sound
			sound = 1;				// make sound
			unit_delay();			// three units for DAH
			unit_delay();
			unit_delay();
			gp_copy = gp_copy & ~TX_ON | SOUND_ON;	// stop TX, leave buzzer line to 1, key open
			GPIO = gp_copy;
			TRIS = READ_SW;			// back to key read mode
			sound = 0;
			unit_delay();
				}
			}

		if(straight == 1 && gk == 1)	// Manual mode, started with both keys down
			goto Manual;


Golden_Key:

		if(DI == 0)					// generate DI
			{
			gp_copy = gp_copy | TX_ON & ~SOUND_ON;	// start TX, 0 to buzzer line = less click
			GPIO = gp_copy;
			TRIS = BUZZ_OUT;		// switch GP2 for sound output
			TMR0 = 0;				// sync timer for sound
			sound = 1;				// make sound
			unit_delay();			// 1 unit fir DI
			gp_copy = gp_copy & ~TX_ON | SOUND_ON;	// stop TX, leave buzzer line to 1, key open
			GPIO = gp_copy;
			TRIS = READ_SW;			// back to key read mode
			sound = 0;
			unit_delay();
			}

		if(DI == 0 && gk == 1)		// DI precedence, Golden Key type action
			goto Golden_Key;

		}							// end for(;;)
}									// end main()
